@@ -1075,6 +1075,7 @@ typedef struct {
   guint32 user_time;
 
   EphyWindow *window;
+  gulong destroy_id;
   gboolean is_first_window;
   gint active_tab;
 
@@ -1099,8 +1100,11 @@ static void
 session_parser_context_free (SessionParserContext *context)
 {
   g_object_unref (context->session);
-  if (context->window)
-    g_object_unref (context->window);
+
+  if (context->window) {
+    /* This can only happen if the session state is malformed. */
+    g_signal_handler_disconnect (context->window, context->destroy_id);
+  }
 
   g_free (context);
 }
@@ -1113,9 +1117,13 @@ session_parse_window (SessionParserContext  *context,
   GdkRectangle geometry = { -1, -1, 0, 0 };
   guint i;
 
-  g_assert (!context->window);
+  if (context->window) {
+    /* This should only happen if the session state is malformed. */
+    return;
+  }
+
   context->window = ephy_window_new ();
-  g_object_ref (context->window);
+  context->destroy_id = g_signal_connect (context->window, "destroy", G_CALLBACK (gtk_widget_destroyed), &context->window);
 
   for (i = 0; names[i]; i++) {
     gulong int_value;
@@ -1158,6 +1166,13 @@ session_parse_embed (SessionParserContext  *context,
   gboolean is_pin = FALSE;
   guint i;
 
+  if (!context->window) {
+    /* This can happen if the session is malformed, or if the window is
+     * destroyed before the session finishes loading.
+     */
+    return;
+  }
+
   notebook = ephy_window_get_notebook (context->window);
 
   for (i = 0; names[i]; i++) {
@@ -1287,6 +1302,13 @@ session_end_element (GMarkupParseContext  *ctx,
     GtkWidget *notebook;
     EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
+    if (!context->window) {
+      /* This can happen if the session is malformed, or if the window is
+       * destroyed before the session finishes loading.
+       */
+      return;
+    }
+
     notebook = ephy_window_get_notebook (context->window);
     gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), context->active_tab);
 
@@ -1301,6 +1323,7 @@ session_end_element (GMarkupParseContext  *ctx,
 
     ephy_embed_shell_restored_window (shell);
 
+    g_clear_signal_handler (&context->destroy_id, context->window);
     context->window = NULL;
     context->active_tab = 0;
     context->is_first_window = FALSE;
@@ -1745,3 +1768,9 @@ ephy_session_clear (EphySession *session)
 
   ephy_session_save (session);
 }
+
+gboolean
+ephy_session_is_closing (EphySession *session)
+{
+  return session->closing;
+}
